@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -17,39 +17,33 @@
 #
 """ Options module """
 
-version_string = """\
-Nuitka V0.5.19
-Copyright (C) 2016 Kay Hayen."""
-
 import logging
+import os
 import sys
 from optparse import SUPPRESS_HELP, OptionGroup, OptionParser
 
-from nuitka.PythonVersions import (
-    getSupportedPythonVersions,
-    getSupportedPythonVersionStr,
-    python_version_str
-)
+from nuitka.PythonVersions import getSupportedPythonVersions
 from nuitka.utils import Utils
+from nuitka.Version import getNuitkaVersion
 
 # Indicator if we were called as "nuitka-run" in which case we assume some
 # other defaults and work a bit different with parameters.
-is_nuitka_run = Utils.basename(sys.argv[0]).lower().startswith("nuitka-run")
-
-def getVersion():
-    return version_string.split()[1][1:]
-
-def getYear():
-    return int(version_string.split()[4])
+is_nuitka_run = os.path.basename(sys.argv[0]).lower().endswith("-run")
 
 if not is_nuitka_run:
-    usage = "usage: %prog [--module] [--execute] [options] main_module.py"
+    usage = "usage: %prog [--module] [--run] [options] main_module.py"
 else:
     usage = "usage: %prog [options] main_module.py"
 
 parser = OptionParser(
     usage   = usage,
-    version = getVersion()
+    version = '\n'.join(
+        (
+            getNuitkaVersion(),
+            sys.version,
+            sys.executable
+        )
+    )
 )
 
 # This option is obsolete, and module should be used.
@@ -78,23 +72,11 @@ parser.add_option(
     help    = """\
 Enable standalone mode in build. This allows you to transfer the created binary
 to other machines without it relying on an existing Python installation. It
-implies these options: "--recurse-all --recurse-stdlib". You may also want
-to use "--python-flag=no_site" to avoid the "site.py" module. Defaults to
-off.""",
+implies these option: "--recurse-all". You may also want to use
+"--python-flag=no_site" to avoid the "site.py" module, which can save a lot
+of code dependencies. Defaults to off.""",
 )
 
-parser.add_option(
-    "--nofreeze-stdlib",
-    action  = "store_false",
-    dest    = "freeze_stdlib",
-    default = True,
-    help    = """\
-In standalone mode by default all modules of standard library will be frozen
-as bytecode. This compiles them all and as a result compilation time will
-increase very much.
-""",
-    )
-
 
 parser.add_option(
     "--python-version",
@@ -102,14 +84,29 @@ parser.add_option(
     dest    = "python_version",
     choices = getSupportedPythonVersions(),
     default = None,
-    help    = """\
-Major version of Python to be used, one of %s.
-Defaults to what you run Nuitka with (currently %s)""" % (
-       getSupportedPythonVersionStr(),
-       python_version_str
-    )
+    help    = SUPPRESS_HELP
+#               """\
+# Major version of Python to be used, one of %s.
+# Defaults to what you run Nuitka with (currently %s)""" % (
+#        getSupportedPythonVersionStr(),
+#        python_version_str
+#     )
 )
 
+if os.name == "nt":
+    parser.add_option(
+        "--python-arch",
+        action  = "store",
+        dest    = "python_arch",
+        choices = ("x86", "x86_64"),
+        default = None,
+        help    = """\
+Architecture of Python to use. One of "x86" or "x86_64".
+Defaults to what you run Nuitka with (currently "%s").""" % (
+           Utils.getArchitecture()
+        )
+    )
+
 parser.add_option(
     "--python-debug", "--python-dbg",
     action  = "store_true",
@@ -128,11 +125,21 @@ parser.add_option(
     help    = """\
 Python flags to use. Default uses what you are using to run Nuitka, this
 enforces a specific mode. These are options that also exist to standard
-Python executable. Currently supported: "-S" (alias nosite),
-"static_hashes" (not use Randomization), "no_warnings" (do not give
-Python runtime warnings). Default empty."""
+Python executable. Currently supported: "-S" (alias "nosite"),
+"static_hashes" (not use hash randomization), "no_warnings" (do not give
+Python runtime warnings), "-O" (alias "noasserts"). Default empty."""
 )
 
+parser.add_option(
+    "--python-for-scons", "--python2-for-scons",
+    action  = "store",
+    dest    = "python_scons",
+    default = None,
+    help    = """\
+If using Python3.3 or Python3.4, provide the path of a Python binary to use
+for Scons. Otherwise Nuitka can use what you run Nuitka with or a "scons"
+binary that is found in PATH, or a Python installation from Windows registry."""
+)
 
 parser.add_option(
     "--warn-implicit-exceptions",
@@ -140,7 +147,56 @@ parser.add_option(
     dest    = "warn_implicit_exceptions",
     default = False,
     help    = """\
-Given warnings for implicit exceptions detected at compile time.""",
+Enable warnings for implicit exceptions detected at compile time.""",
+)
+
+parser.add_option(
+    "--warn-unusual-code",
+    action  = "store_true",
+    dest    = "warn_unusual_code",
+    default = False,
+    help    = """\
+Enable warnings for unusual code detected at compile time.""",
+)
+
+parser.add_option(
+    "--assume-yes-for-downloads",
+    action  = "store_true",
+    dest    = "assume_yes_for_downloads",
+    default = False,
+    help    = """\
+Allow Nuitka to download code if necessary, e.g. dependency walker on Windows.""",
+)
+
+
+include_group = OptionGroup(
+    parser,
+    "Control the inclusion of modules and packages."
+)
+
+include_group.add_option(
+    "--include-package",
+    action  = "append",
+    dest    = "include_packages",
+    metavar = "PACKAGE",
+    default = [],
+    help    = """\
+Include a whole package. Give as a Python namespace, e.g. ``some_package.sub_package``
+and Nuitka will then find it and include it and all the modules found below that
+disk location in the binary or extension module it creates, and make it available
+for import by the code. Default empty."""
+)
+
+include_group.add_option(
+    "--include-module",
+    action  = "append",
+    dest    = "include_modules",
+    metavar = "MODULE",
+    default = [],
+    help    = """\
+Include a single module. Give as a Python namespace, e.g. ``some_package.some_module``
+and Nuitka will then find it and include it in the binary or extension module
+it creates, and make it available for import by the code. Default empty."""
 )
 
 
@@ -156,7 +212,8 @@ recurse_group.add_option(
     dest    = "recurse_stdlib",
     default = False,
     help    = """\
-Also descend into imported modules from standard library. Defaults to off."""
+Also descend into imported modules from standard library. This will increase
+the compilation time by a lot. Defaults to off."""
 )
 
 recurse_group.add_option(
@@ -300,11 +357,21 @@ codegen_group = OptionGroup(
 codegen_group.add_option(
     "--improved", "--enhanced",
     action  = "store_true",
-    dest    = "improved",
+    dest    = "disabled",
     default = False,
+    help    = SUPPRESS_HELP,
+)
+
+codegen_group.add_option(
+    "--full-compat",
+    action  = "store_false",
+    dest    = "improved",
+    default = True,
     help    = """\
-Allow minor deviations from CPython behavior, e.g. better tracebacks, which
-are not really incompatible, but different.""",
+Enforce absolute compatibility with CPython. Do not even allow minor
+deviations from CPython behavior, e.g. better tracebacks, which are
+not really incompatible, but different. This is intended for tests
+only and should not be necessary for normal use.""",
 )
 
 codegen_group.add_option(
@@ -336,20 +403,11 @@ compatibility reasons, the "__file__" value will always have ".py" suffix
 independent of what it really is."""
 )
 
-codegen_group.add_option(
-    "--no-optimization",
-    action  = "store_true",
-    dest    = "no_optimize",
-    default = False,
-    help    = SUPPRESS_HELP
-# """Disable all unnecessary optimizations on Python level. Defaults to off."""
-)
-
 parser.add_option_group(codegen_group)
 
 outputdir_group = OptionGroup(
     parser,
-    "Output directory choices"
+    "Output choices"
 )
 
 outputdir_group.add_option(
@@ -359,8 +417,9 @@ outputdir_group.add_option(
     metavar = "DIRECTORY",
     default = "",
     help    = """\
-Specify where intermediate and final output files should be put. DIRECTORY will
-be populated with C++ files, object files, etc. Defaults to current directory.
+Specify where intermediate and final output files should be put. The DIRECTORY
+will be populated with C files, object files, etc.
+Defaults to current directory.
 """
 )
 
@@ -374,6 +433,18 @@ Removes the build directory after producing the module or exe file.
 Defaults to off."""
 )
 
+outputdir_group.add_option(
+    "--no-pyi-file",
+    action  = "store_false",
+    dest    = "pyi_file",
+    default = True,
+    help    = """\
+Do not create a ".pyi" file for extension modules created by Nuitka.
+Defaults to off."""
+)
+
+
+
 parser.add_option_group(outputdir_group)
 
 
@@ -437,36 +508,37 @@ Defaults to off."""
 )
 
 debug_group.add_option(
-    "--recompile-c++-only",
+    "--recompile-c-only",
     action  = "store_true",
-    dest    = "recompile_cpp_only",
+    dest    = "recompile_c_only",
     default = False,
     help    = """\
-Take existing files and compile them again.Allows compiling edited C++ files
-with the C++ compiler for quick debugging changes to the generated source.
+Take existing files and compile them again. Allows compiling edited C files
+with the C compiler for quick debugging changes to the generated source.
 Defaults to off. Depends on compiling Python source to determine which files it
 should look at."""
 )
 
 debug_group.add_option(
-    "--generate-c++-only",
+    "--generate-c-only",
     action  = "store_true",
-    dest    = "generate_cpp_only",
+    dest    = "generate_c_only",
     default = False,
     help    = """\
-Generate only C++ source code, and do not compile it to binary or module. This
+Generate only C source code, and do not compile it to binary or module. This
 is for debugging and code coverage analysis that doesn't waste CPU. Defaults to
 off."""
 )
 
 debug_group.add_option(
     "--experimental",
-    action  = "store_true",
+    action  = "append",
     dest    = "experimental",
-    default = False,
+    default = [],
     help    = """\
 Use features declared as 'experimental'. May have no effect if no experimental
-features are present in the code. Defaults to off."""
+features are present in the code. Uses secret tags (check source) per
+experimented feature."""
 )
 
 debug_group.add_option(
@@ -477,6 +549,30 @@ debug_group.add_option(
     help    = SUPPRESS_HELP
 )
 
+if os.name == "nt":
+    debug_group.add_option(
+        "--disable-dll-dependency-cache",
+        action  = "store_true",
+        dest    = "no_dependency_cache",
+        default = False,
+        help    = """\
+Disable the dependency walker cache. Will result in much longer times to create
+the distribution folder, but might be used in case the cache is suspect to cause
+errors.
+"""
+    )
+
+    debug_group.add_option(
+        "--force-dll-dependency-cache-update",
+        action  = "store_true",
+        dest    = "update_dependency_cache",
+        default = False,
+        help    = """\
+For an update of the dependency walker cache. Will result in much longer times
+to create the distribution folder, but might be used in case the cache is suspect
+to cause errors or known to need an update.
+"""
+    )
 
 # This is for testing framework, "coverage.py" hates to loose the process. And
 # we can use it to make sure it's not done unknowingly.
@@ -491,13 +587,13 @@ parser.add_option(
 
 parser.add_option_group(debug_group)
 
-cpp_compiler_group = OptionGroup(
+c_compiler_group = OptionGroup(
     parser,
-    "Backend C++ compiler choice"
+    "Backend C compiler choice"
 )
 
 
-cpp_compiler_group.add_option(
+c_compiler_group.add_option(
     "--clang",
     action  = "store_true",
     dest    = "clang",
@@ -507,7 +603,7 @@ Enforce the use of clang (needs clang 3.2 or higher).
 Defaults to off."""
 )
 
-cpp_compiler_group.add_option(
+c_compiler_group.add_option(
     "--mingw",
     action  = "store_true",
     dest    = "mingw",
@@ -517,7 +613,7 @@ Enforce the use of MinGW on Windows.
 Defaults to off."""
 )
 
-cpp_compiler_group.add_option(
+c_compiler_group.add_option(
     "--msvc",
     action  = "store",
     dest    = "msvc",
@@ -528,18 +624,18 @@ are e.g. 9.0, 9.0exp, specify an illegal value for a list of installed
 compilers. Defaults to the most recent version."""
 )
 
-cpp_compiler_group.add_option(
+c_compiler_group.add_option(
     "-j", "--jobs",
     action  = "store",
     dest    = "jobs",
     metavar = 'N',
     default = Utils.getCoreCount(),
     help    = """\
-Specify the allowed number of parallel C++ compiler jobs. Defaults to the
+Specify the allowed number of parallel C compiler jobs. Defaults to the
 system CPU count.""",
 )
 
-cpp_compiler_group.add_option(
+c_compiler_group.add_option(
     "--lto",
     action  = "store_true",
     dest    = "lto",
@@ -549,7 +645,7 @@ Use link time optimizations if available and usable (g++ 4.6 and higher).
 Defaults to off."""
 )
 
-parser.add_option_group(cpp_compiler_group)
+parser.add_option_group(c_compiler_group)
 
 tracing_group = OptionGroup(
     parser,
@@ -664,80 +760,162 @@ this detection code is run in vain once you are certain of which plug-ins to
 use. Defaults to off."""
 )
 
+plugin_group.add_option(
+    "--plugin-list",
+    action  = "store_true",
+    dest    = "list_plugins",
+    default = False,
+    help    = """\
+Show list of all available plugins and exit. Defaults to off."""
+)
+
+
 parser.add_option_group(plugin_group)
 
-# First, isolate the first non-option arguments.
-if is_nuitka_run:
-    count = 0
 
-    for count, arg in enumerate(sys.argv):
-        if count == 0:
-            continue
+options = None
+positional_args = None
+extra_args = []
 
-        if arg[0] != '-':
-            break
+def parseArgs():
+    # singleton with many cases, pylint: disable=global-statement,too-many-branches,too-many-statements
+    global options, positional_args, extra_args
 
-        # Treat "--" as a terminator.
-        if arg == "--":
-            count += 1
-            break
+    # First, isolate the first non-option arguments.
+    if is_nuitka_run:
+        count = 0
 
-    if count > 0:
-        extra_args = sys.argv[count+1:]
-        sys.argv = sys.argv[0:count+1]
-else:
-    extra_args = []
+        for count, arg in enumerate(sys.argv):
+            if count == 0:
+                continue
 
-options, positional_args = parser.parse_args()
+            if arg[0] != '-':
+                break
 
-if not positional_args:
-    parser.print_help()
+            # Treat "--" as a terminator.
+            if arg == "--":
+                count += 1
+                break
 
-    sys.exit("""
-Error, need positional argument with python module or main program.""")
+        if count > 0:
+            extra_args = sys.argv[count+1:]
+            sys.argv = sys.argv[0:count+1]
 
-if not options.immediate_execution and len(positional_args) > 1:
-    parser.print_help()
+    options, positional_args = parser.parse_args()
 
-    sys.exit("""
-Error, need only one positional argument unless --run is specified.""")
+    if shallListPlugins():
+        from nuitka.plugins.Plugins import listPlugins
+        listPlugins()
 
-if options.verbose:
-    logging.getLogger().setLevel(logging.DEBUG)
-else:
-    logging.getLogger().setLevel(logging.INFO)
+    if not positional_args:
+        parser.print_help()
+
+        sys.exit("""
+    Error, need positional argument with python module or main program.""")
+
+    if not options.immediate_execution and len(positional_args) > 1:
+        parser.print_help()
+
+        sys.exit("""
+    Error, need only one positional argument unless "--run" is specified to
+    pass them to the compiled program execution.""")
+
+    if options.verbose:
+        logging.getLogger().setLevel(logging.DEBUG)
+    else:
+        logging.getLogger().setLevel(logging.INFO)
+
+    # Standalone mode implies an executable, not importing "site" module, which is
+    # only for this machine, recursing to all modules, and even including the
+    # standard library.
+    if options.is_standalone:
+        if not options.executable:
+            sys.exit("""\
+Error, conflicting options, cannot make standalone module, only executable.""")
+
+        options.recurse_all = True
+
+        if Utils.getOS() == "NetBSD":
+            logging.warning("Standalone mode on NetBSD is not functional, due to $ORIGIN linkage not being supported.")
+
+    for any_case_module in getShallFollowModules():
+        if any_case_module.startswith('.'):
+            bad = True
+        else:
+            for char in "/\\:":
+                if  char in any_case_module:
+                    bad = True
+                    break
+            else:
+                bad = False
+
+        if bad:
+            sys.exit(
+                """\
+Error, '--recurse-to' takes only module names, not directory path '%s'.""" % \
+                any_case_module
+            )
+
+    for no_case_module in getShallFollowInNoCase():
+        if no_case_module.startswith('.'):
+            bad = True
+        else:
+            for char in "/\\:":
+                if  char in no_case_module:
+                    bad = True
+                    break
+            else:
+                bad = False
+
+        if bad:
+            sys.exit(
+                """\
+Error, '--recurse-not-to' takes only module names, not directory path '%s'.""" % \
+                no_case_module
+            )
+
+    scons_python = getPythonPathForScons()
+
+    if scons_python is not None and not os.path.exists(scons_python):
+        sys.exit("Error, no such Python2 binary '%s'." % scons_python)
+
+
+def isVerbose():
+    return options.verbose
 
-# Standalone mode implies an executable, not importing "site" module, which is
-# only for this machine, recursing to all modules, and even including the
-# standard library.
-if options.is_standalone:
-    options.executable = True
-    options.recurse_all = True
 
 def shallTraceExecution():
     return options.trace_execution
 
+
 def shallExecuteImmediately():
     return options.immediate_execution
 
+
 def shallRunInDebugger():
     return options.debugger
 
+
 def shallDumpBuiltTreeXML():
     return options.dump_xml
 
+
 def shallDisplayBuiltTree():
     return options.display_tree
 
-def shallOnlyExecCppCall():
-    return options.recompile_cpp_only
 
-def shallNotDoExecCppCall():
-    return options.generate_cpp_only
+def shallOnlyExecCCompilerCall():
+    return options.recompile_c_only
+
+
+def shallNotDoExecCCompilerCall():
+    return options.generate_c_only
+
 
 def shallHaveStatementLines():
     return options.statement_lines
 
+
 def getFileReferenceMode():
     if options.file_reference_mode is None:
         value = ("runtime"
@@ -748,188 +926,307 @@ def getFileReferenceMode():
 
     return value
 
+
 def shallMakeModule():
     return not options.executable
 
-def shallFollowStandardLibrary():
-    return options.recurse_stdlib
 
-def shallFollowNoImports():
-    return options.recurse_none
+def shallCreatePyiFile():
+    return options.pyi_file
 
-def shallFollowAllImports():
-    return options.recurse_all
-
-def getShallFollowModules():
-    return sum([ x.split(',') for x in options.recurse_modules ], [])
 
 def isAllowedToReexecute():
     return options.allow_reexecute
 
-for any_case_module in getShallFollowModules():
-    if any_case_module.startswith('.'):
-        bad = True
-    else:
-        for char in "/\\:":
-            if  char in any_case_module:
-                bad = True
-                break
-        else:
-            bad = False
 
-    if bad:
-        sys.exit("""
-Error, '--recurse-to' takes only module names, not directory path '%s'.""" % \
-any_case_module)
+def shallFollowStandardLibrary():
+    return options.recurse_stdlib
+
+
+def shallFollowNoImports():
+    return options.recurse_none
+
+
+def shallFollowAllImports():
+    return options.recurse_all
+
+
+def _splitShellPattern(value):
+    return value.split(',') if '{' not in value else [value]
+
 
 def getShallFollowInNoCase():
-    return sum([ x.split(',') for x in options.recurse_not_modules ], [])
+    return sum(
+        [_splitShellPattern(x) for x in options.recurse_not_modules ],
+        []
+    )
 
-for no_case_module in getShallFollowInNoCase():
-    if no_case_module.startswith('.'):
-        bad = True
-    else:
-        for char in "/\\:":
-            if  char in no_case_module:
-                bad = True
-                break
-        else:
-            bad = False
 
-    if bad:
-        sys.exit("""
-Error, '--recurse-not-to' takes only module names, not directory path '%s'.""" % \
-no_case_module)
+def getShallFollowModules():
+    return sum(
+        [_splitShellPattern(x) for x in options.recurse_modules + options.include_modules + options.include_packages],
+        []
+    )
 
 
 def getShallFollowExtra():
-    return sum([ x.split(',') for x in options.recurse_extra ], [])
+    return sum(
+        [_splitShellPattern(x) for x in options.recurse_extra],
+        []
+    )
+
 
 def getShallFollowExtraFilePatterns():
-    return sum([ x.split(',') for x in options.recurse_extra_files ], [])
+    return sum(
+        [_splitShellPattern(x) for x in options.recurse_extra_files],
+        []
+    )
+
+
+def getMustIncludeModules():
+    return sum(
+        [_splitShellPattern(x) for x in options.include_modules],
+        []
+    )
+
+
+def getMustIncludePackages():
+    return sum(
+        [_splitShellPattern(x) for x in options.include_packages ],
+        []
+    )
+
 
 def shallWarnImplicitRaises():
     return options.warn_implicit_exceptions
 
+
+def shallWarnUnusualCode():
+    return options.warn_unusual_code
+
+
+def assumeYesForDownloads():
+    return options.assume_yes_for_downloads
+
+
 def isDebug():
-    return options.debug or options.debugger
+    return options is not None and (options.debug or options.debugger)
+
 
 def isPythonDebug():
     return options.python_debug or sys.flags.debug
 
-def isOptimize():
-    return not options.no_optimize
 
 def isUnstripped():
     return options.unstripped or options.profile
 
+
 def isProfile():
     return options.profile
 
-def shouldCreateGraph():
+
+def shallCreateGraph():
     return options.graph
 
+
 def getOutputPath(path):
     if options.output_dir:
-        return Utils.normpath(Utils.joinpath(options.output_dir, path))
+        return os.path.normpath(os.path.join(options.output_dir, path))
     else:
         return path
 
+
 def getOutputDir():
     return options.output_dir if options.output_dir else '.'
 
+
 def getPositionalArgs():
     return tuple(positional_args)
 
+
 def getMainArgs():
     return tuple(extra_args)
 
+
 def shallOptimizeStringExec():
     return False
 
+
 def shallClearPythonPathEnvironment():
     return not options.keep_pythonpath
 
+
 def isShowScons():
     return options.show_scons
 
+
 def getJobLimit():
     return int(options.jobs)
 
+
 def isLto():
     return options.lto
 
+
 def isClang():
     return options.clang
 
+
 def isMingw():
     return options.mingw
 
+
 def getMsvcVersion():
     return options.msvc
 
+
 def shallDisableConsoleWindow():
     return options.win_disable_console
 
+
 def isFullCompat():
     return not options.improved
 
+
 def isShowProgress():
     return options.show_progress
 
+
 def isShowMemory():
-    return options.show_memory
+    return options is not None and options.show_memory
+
 
 def isShowInclusion():
     return options.show_inclusion
 
+
 def isRemoveBuildDir():
-    return options.remove_build and not options.generate_cpp_only
+    return options.remove_build and not options.generate_c_only
+
 
 def getIntendedPythonVersion():
     return options.python_version
 
-def isExperimental():
-    return hasattr(options, "experimental") and options.experimental
+
+def getIntendedPythonArch():
+    return options.python_arch
+
+
+def isExperimental(indication):
+    """ Are experimental features to be enabled."""
+
+    return hasattr(options, "experimental") and indication in options.experimental
+
+
+def getExperimentalIndications():
+    if hasattr(options, "experimental"):
+        return options.experimental
+    else:
+        return ()
+
 
 def shallExplainImports():
-    return options.explain_imports
+    return options is not None and options.explain_imports
+
 
 def isStandaloneMode():
     return options.is_standalone
 
+
 def getIconPath():
     return options.icon_path
 
+_python_flags = None
+
 def getPythonFlags():
+    # singleton, pylint: disable=global-statement
+    global _python_flags
+
+    if _python_flags is None:
+        _python_flags = set()
+
+        for parts in options.python_flags:
+            for part in parts.split(','):
+                if part in ("-S", "nosite", "no_site"):
+                    _python_flags.add("no_site")
+                elif part in ("static_hashes", "norandomization",
+                              "no_randomization"):
+                    _python_flags.add("no_randomization")
+                elif part in ("-v", "trace_imports", "trace_import"):
+                    _python_flags.add("trace_imports")
+                elif part in ("no_warnings", "nowarnings"):
+                    _python_flags.add("no_warnings")
+                elif part in ("-O", "no_asserts", "noasserts"):
+                    _python_flags.add("no_asserts")
+                else:
+                    # Do not warn before executing in final context.
+                    if "PYTHONHASHSEED" in os.environ:
+                        logging.warning("Unsupported flag '%s'.", part)
+
+    return _python_flags
+
+
+def shallFreezeAllStdlib():
+    return not shallFollowStandardLibrary()
+
+
+def shallNotUseDependsExeCachedResults():
+    return options.no_dependency_cache or options.update_dependency_cache
+
+
+def shallNotStoreDependsExeCachedResults():
+    return options.no_dependency_cache
+
+
+def shallListPlugins():
+    return options is not None and options.list_plugins
+
+
+def getPluginsEnabled():
+    """ Return the names of plugin that were enabled.
+
+    """
+    result = set()
+
+    if options:
+        for plugin_enabled in options.plugins_enabled:
+            result.add(plugin_enabled.split('=',1)[0])
+
+    return tuple(result)
+
+
+def getPluginOptions(plugin_name):
+    """ Return the options provided for a specific plugin.
+
+    """
     result = []
 
-    for part in options.python_flags:
-        if part in ("-S", "nosite", "no_site"):
-            result.append("no_site")
-        elif part in ("static_hashes", "norandomization", "no_randomization"):
-            result.append("no_randomization")
-        elif part in ("-v", "trace_imports", "trace_import"):
-            result.append("trace_imports")
-        elif part in ("no_warnings", "nowarnings"):
-            result.append("no_warnings")
-        else:
-            logging.warning("Unsupported flag '%s'.", part)
+    if options:
+        for plugin_enabled in options.plugins_enabled:
+            if '=' not in plugin_enabled:
+                continue
+
+            name, args = plugin_enabled.split('=',1)
+
+            if name == plugin_name:
+                result.extend(args.split(','))
 
     return result
 
-def shallFreezeAllStdlib():
-    return options.freeze_stdlib
-
-def getPluginsEnabled():
-    return options.plugins_enabled
 
 def getPluginsDisabled():
-    return options.plugins_disabled
+    """ Return the names of plugin that were disabled.
+
+    """
+    if not options:
+        return ()
+
+    return tuple(set(options.plugins_disabled))
+
 
 def shallDetectMissingPlugins():
-    return options.detect_missing_plugins
+    return options is not None and options.detect_missing_plugins
 
-def getPluginOptions(plugin_name):
-    # TODO: This should come from command line, pylint: disable=W0613
-    return {}
+
+def getPythonPathForScons():
+    return options.python_scons
