@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -58,10 +58,32 @@ def getErrorExitReleaseCode(context):
     return temp_release
 
 
-def getErrorExitBoolCode(condition, emit, context,
+def getFrameVariableTypeDescriptionCode(context):
+
+    type_description = context.getFrameVariableTypeDescription()
+
+    if type_description:
+        return '%s = "%s";' % (
+            context.getFrameVariableTypeDescriptionName(),
+            type_description,
+        )
+    else:
+        return ""
+
+
+def getErrorExitBoolCode(condition, emit, context, release_names = (),
+                         release_name = None,
                          needs_check = True, quick_exception = None):
     assert not condition.endswith(';')
 
+    if release_names:
+        getReleaseCodes(release_names, emit, context)
+        assert not release_name
+
+    if release_name is not None:
+        getReleaseCode(release_name, emit, context)
+        assert not release_names
+
     if not needs_check:
         getAssertionCode("!(%s)" % condition, emit)
         return
@@ -72,13 +94,16 @@ def getErrorExitBoolCode(condition, emit, context,
         emit(
             indented(
                 template_error_catch_quick_exception % {
-                    "condition"        : condition,
-                    "exception_exit"   : context.getExceptionEscape(),
-                    "quick_exception"  : getExceptionIdentifier(quick_exception),
-                    "release_temps"    : indented(
+                    "condition"            : condition,
+                    "exception_exit"       : context.getExceptionEscape(),
+                    "quick_exception"      : getExceptionIdentifier(quick_exception),
+                    "release_temps"        : indented(
                         getErrorExitReleaseCode(context)
                     ),
-                    "line_number_code" : indented(
+                    "var_description_code" : indented(
+                        getFrameVariableTypeDescriptionCode(context),
+                    ),
+                    "line_number_code"     : indented(
                         getErrorLineNumberUpdateCode(context)
                     )
                 },
@@ -94,6 +119,9 @@ def getErrorExitBoolCode(condition, emit, context,
                     "release_temps"    : indented(
                         getErrorExitReleaseCode(context)
                     ),
+                    "var_description_code": indented(
+                        getFrameVariableTypeDescriptionCode(context),
+                    ),
                     "line_number_code" : indented(
                         getErrorLineNumberUpdateCode(context)
                     )
@@ -103,16 +131,18 @@ def getErrorExitBoolCode(condition, emit, context,
         )
 
 
-def getErrorExitCode(check_name, emit, context, quick_exception = None, needs_check = True):
-    if needs_check:
-        getErrorExitBoolCode(
-            condition       = "%s == NULL" % check_name,
-            quick_exception = quick_exception,
-            emit            = emit,
-            context         = context
-        )
-    else:
-        getAssertionCode("%s != NULL" % check_name, emit)
+def getErrorExitCode(check_name, emit, context, release_names = (),
+                     release_name = None, quick_exception = None,
+                     needs_check = True):
+    getErrorExitBoolCode(
+        condition       = "%s == NULL" % check_name,
+        release_names   = release_names,
+        release_name    = release_name,
+        needs_check     = needs_check,
+        quick_exception = quick_exception,
+        emit            = emit,
+        context         = context
+    )
 
 
 def getErrorFormatExitBoolCode(condition, exception, args, emit, context):
@@ -120,7 +150,7 @@ def getErrorFormatExitBoolCode(condition, exception, args, emit, context):
 
     context.markAsNeedsExceptionVariables()
 
-    if len(args) == 1:
+    if len(args) == 1 and type(args[0]) is str:
         from .ConstantCodes import getModuleConstantCode
 
         set_exception = [
@@ -132,7 +162,15 @@ def getErrorFormatExitBoolCode(condition, exception, args, emit, context):
             "exception_tb = NULL;"
         ]
     else:
-        assert False, args
+        set_exception = [
+            "exception_type = %s;" % exception,
+            "Py_INCREF( exception_type );",
+            "exception_value = Py%s_FromFormat( %s );" % (
+                "String" if python_version < 300 else "Unicode",
+                ", ".join( '"%s"' % arg for arg in args )
+            ),
+            "exception_tb = NULL;"
+        ]
 
 
     if python_version >= 300:
@@ -150,7 +188,7 @@ def getErrorFormatExitBoolCode(condition, exception, args, emit, context):
                 "NORMALIZE_EXCEPTION( &exception_type, &exception_value, &exception_tb );"
             )
             set_exception.append(
-                "CHAIN_EXCEPTION( exception_type, exception_value );"
+                "CHAIN_EXCEPTION( exception_value );"
             )
 
     emit(
@@ -161,17 +199,22 @@ def getErrorFormatExitBoolCode(condition, exception, args, emit, context):
             "release_temps"    : indented(
                 getErrorExitReleaseCode(context)
             ),
+            "var_description_code" : indented(
+                getFrameVariableTypeDescriptionCode(context),
+            ),
             "line_number_code" : indented(
                 getErrorLineNumberUpdateCode(context)
             )
         }
     )
 
+
 def getErrorVariableDeclarations():
     return (
-        "PyObject *exception_type = NULL, *exception_value = NULL;",
+        "PyObject *exception_type = NULL;",
+        "PyObject *exception_value = NULL;",
         "PyTracebackObject *exception_tb = NULL;",
-        "NUITKA_MAY_BE_UNUSED int exception_lineno = -1;"
+        "NUITKA_MAY_BE_UNUSED int exception_lineno = 0;"
     )
 
 def getExceptionKeeperVariableNames(keeper_index):
@@ -199,7 +242,7 @@ def getExceptionKeeperVariableNames(keeper_index):
         ),
         "NUITKA_MAY_BE_UNUSED int exception_keeper_lineno_%d%s;" % (
             keeper_index,
-            "= -1" if debug else ""
+            " = 0" if debug else ""
         )
     )
 
@@ -260,7 +303,7 @@ def getReleaseCodes(release_names, emit, context):
 def getMustNotGetHereCode(reason, context, emit):
     getCommentCode(reason, emit)
 
-    provider = context.getOwner()
+    provider = context.getEntryPoint()
 
     emit(
         "NUITKA_CANNOT_GET_HERE( %(function_identifier)s );" % {
@@ -268,15 +311,72 @@ def getMustNotGetHereCode(reason, context, emit):
         }
     )
 
-    if provider.isExpressionFunctionBody() and not provider.isGenerator():
-        emit("return NULL;")
-    elif provider.isPythonModule():
-        emit("PyErr_SetObject( PyExc_RuntimeError, Py_None );")
+    if provider.isExpressionGeneratorObjectBody():
+        emit("return;")
+    elif provider.isExpressionCoroutineObjectBody():
+        emit("return;")
+    elif provider.isExpressionAsyncgenObjectBody():
+        emit("return;")
+    elif provider.isCompiledPythonModule():
         emit("return MOD_RETURN_VALUE( NULL );")
     else:
-        emit("PyErr_SetObject( PyExc_RuntimeError, Py_None );")
-        emit("return;")
+        emit("return NULL;")
 
 
 def getAssertionCode(check, emit):
     emit("assert( %s );" % check)
+
+
+def getCheckObjectCode(check_name, emit):
+    emit("CHECK_OBJECT( %s );" % check_name)
+
+
+def getLocalVariableReferenceErrorCode(variable, condition, emit, context):
+    if variable.getOwner() is not context.getOwner():
+        getErrorFormatExitBoolCode(
+            condition = condition,
+            exception = "PyExc_NameError",
+            args      = (
+                """\
+free variable '%s' referenced before assignment in enclosing scope""",
+                variable.getName()
+            ),
+            emit      = emit,
+            context   = context
+        )
+    else:
+        getErrorFormatExitBoolCode(
+            condition = condition,
+            exception = "PyExc_UnboundLocalError",
+            args      = (
+                """\
+local variable '%s' referenced before assignment""",
+                variable.getName()
+            ),
+            emit      = emit,
+            context   = context
+        )
+
+
+def getNameReferenceErrorCode(variable_name, condition, emit, context):
+    if python_version < 340:
+        owner = context.getOwner()
+
+        if not owner.isCompiledPythonModule() and \
+           not owner.isExpressionClassBody():
+            error_message = "global name '%s' is not defined"
+        else:
+            error_message = "name '%s' is not defined"
+    else:
+        error_message = "name '%s' is not defined"
+
+    getErrorFormatExitBoolCode(
+        condition = condition,
+        exception = "PyExc_NameError",
+        args      = (
+            error_message,
+            variable_name
+        ),
+        emit      = emit,
+        context   = context
+    )
