@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -19,7 +19,7 @@
 
 """
 
-def getLineNumberUpdateCode(context):
+def getCurrentLineNumberCode(context):
     frame_handle = context.getFrameHandle()
 
     if frame_handle is None:
@@ -27,16 +27,44 @@ def getLineNumberUpdateCode(context):
     else:
         source_ref = context.getCurrentSourceCodeReference()
 
-        if source_ref.shallSetCurrentLine():
-            return "%s->f_lineno = %d;" % (
-                frame_handle,
-                source_ref.getLineNumber()
-            )
-        else:
+        if source_ref.isInternal():
             return ""
+        else:
+            return str(source_ref.getLineNumber())
+
+def getLineNumberUpdateCode(context):
+    lineno_value = getCurrentLineNumberCode(context)
+
+    if lineno_value:
+        frame_handle = context.getFrameHandle()
+
+        return "%s->m_frame.f_lineno = %s;" % (
+            frame_handle,
+            lineno_value
+        )
+    else:
+        return ""
 
 
-def emitLineNumberUpdateCode(context, emit):
+def getErrorLineNumberUpdateCode(context):
+    lineno_value = getCurrentLineNumberCode(context)
+
+    if lineno_value:
+        return "exception_lineno = %s;" % (
+            lineno_value
+        )
+    else:
+        return ""
+
+
+def emitErrorLineNumberUpdateCode(emit, context):
+    update_code = getErrorLineNumberUpdateCode(context)
+
+    if update_code:
+        emit(update_code)
+
+
+def emitLineNumberUpdateCode(emit, context):
     code = getLineNumberUpdateCode(context)
 
     if code:
@@ -47,7 +75,7 @@ def getSetLineNumberCodeRaw(to_name, emit, context):
     assert context.getFrameHandle() is not None
 
     emit(
-        "%s->f_lineno = %s;" % (
+        "%s->m_frame.f_lineno = %s;" % (
             context.getFrameHandle(),
             to_name
         )
@@ -58,7 +86,7 @@ def getLineNumberCode(to_name, emit, context):
     assert context.getFrameHandle() is not None
 
     emit(
-        "%s = %s->f_lineno;"  % (
+        "%s = %s->m_frame.f_lineno;"  % (
             to_name,
             context.getFrameHandle()
         )
