@@ -33,6 +33,8 @@ from .NodeMakingHelpers import (
     wrapExpressionWithNodeSideEffects,
     wrapStatementWithSideEffects
 )
+from .OperatorNodes import ExpressionOperationNOT
+from .StatementNodes import StatementsSequence
 
 
 class ExpressionConditional(ExpressionChildrenHavingBase):
@@ -55,8 +57,6 @@ class ExpressionConditional(ExpressionChildrenHavingBase):
             source_ref = source_ref
         )
 
-        self.merge_traces = None
-
     def getBranches(self):
         return (
             self.getExpressionYes(),
@@ -75,6 +75,8 @@ class ExpressionConditional(ExpressionChildrenHavingBase):
     )
 
     def computeExpressionRaw(self, trace_collection):
+        # This is rather complex stuff, pylint: disable=too-many-branches
+
         # Query the truth value after the expression is evaluated, once it is
         # evaluated in onExpression, it is known.
         trace_collection.onExpression(
@@ -151,11 +153,16 @@ branches."""
         else:
             branch_no_collection = None
 
-        # Merge into parent execution.
-        self.merge_traces = trace_collection.mergeBranches(
-            branch_yes_collection,
-            branch_no_collection
-        )
+        if truth_value is True:
+            trace_collection.replaceBranch(branch_yes_collection)
+        elif truth_value is False:
+            trace_collection.replaceBranch(branch_no_collection)
+        else:
+            # Merge into parent execution.
+            trace_collection.mergeBranches(
+                branch_yes_collection,
+                branch_no_collection
+            )
 
         if truth_value is True:
             return (
@@ -164,7 +171,7 @@ branches."""
                     old_node = condition
                 ),
                 "new_expression",
-                "Conditional expression predicted to yes case"
+                "Conditional expression predicted to yes case."
             )
         elif truth_value is False:
             return (
@@ -173,7 +180,7 @@ branches."""
                     old_node = condition
                 ),
                 "new_expression",
-                "Conditional expression predicted to no case"
+                "Conditional expression predicted to no case."
             )
         else:
             return self, None, None
@@ -244,8 +251,6 @@ class ExpressionConditionalBoolBase(ExpressionChildrenHavingBase):
             source_ref = source_ref
         )
 
-        self.merge_traces = None
-
     getLeft = ExpressionChildrenHavingBase.childGetter(
         "left"
     )
@@ -313,12 +318,10 @@ branches.""" % self.conditional_kind
 
         if branch_yes_collection:
             # Merge into parent execution.
-            self.merge_traces = trace_collection.mergeBranches(
+            trace_collection.mergeBranches(
                 branch_yes_collection,
                 None
             )
-        else:
-            self.merge_traces = None
 
         if truth_value is truth_value_use_left:
             return (
@@ -426,7 +429,6 @@ class StatementConditional(StatementChildrenHavingBase):
             source_ref = source_ref
         )
 
-        self.merge_traces = None
 
     getCondition = StatementChildrenHavingBase.childGetter("condition")
     getBranchYes = StatementChildrenHavingBase.childGetter("yes_branch")
@@ -532,7 +534,6 @@ branches."""
                 BaseException
             )
 
-
         # Query the truth value after the expression is evaluated, once it is
         # evaluated in onExpression, it is known.
         truth_value = condition.getTruthValue()
@@ -545,11 +546,18 @@ branches."""
         # Handle branches that became empty behind our back.
         if yes_branch is not None:
             if not yes_branch.getStatements():
+                yes_branch.finalize()
                 yes_branch = None
+
+                self.setBranchYes(None)
+
         if no_branch is not None:
             if not no_branch.getStatements():
+                no_branch.finalize()
                 no_branch = None
 
+                self.setBranchNo(None)
+
         # Consider to not remove branches that we know won't be taken.
         if yes_branch is not None and truth_value is False:
             trace_collection.signalChange(
@@ -558,6 +566,7 @@ branches."""
                 message    = "Removed conditional branch not taken due to false condition value."
             )
 
+            yes_branch.finalize()
             self.setBranchYes(None)
             yes_branch = None
 
@@ -568,6 +577,7 @@ branches."""
                 message    = "Removed 'else' branch not taken due to true condition value."
             )
 
+            no_branch.finalize()
             self.setBranchNo(None)
             no_branch = None
 
@@ -612,11 +622,17 @@ branches."""
         else:
             branch_no_collection = None
 
-        # Merge into parent execution.
-        self.merge_traces = trace_collection.mergeBranches(
-            branch_yes_collection,
-            branch_no_collection
-        )
+        if truth_value is True:
+            if branch_yes_collection is not None:
+                trace_collection.replaceBranch(branch_yes_collection)
+        elif truth_value is False:
+            if branch_no_collection is not None:
+                trace_collection.replaceBranch(branch_no_collection)
+        else:
+            trace_collection.mergeBranches(
+                branch_yes_collection,
+                branch_no_collection
+            )
 
         # Both branches may have become empty, which case, the statement needs
         # not remain.
@@ -636,9 +652,13 @@ branches."""
                     node       = self
                 )
 
+                del self.parent
+
                 return result, "new_statements", """\
 Both branches have no effect, reduced to evaluate condition."""
             else:
+                self.finalize()
+
                 return None, "new_statements", """\
 Removed conditional statement without effect."""
 
@@ -650,11 +670,15 @@ Removed conditional statement without effect."""
             if truth_value is True:
                 choice = "true"
 
-                new_statement = self.getBranchYes()
+                new_statement = yes_branch
+                if no_branch is not None:
+                    no_branch.finalize()
             else:
                 choice = "false"
 
-                new_statement = self.getBranchNo()
+                new_statement = no_branch
+                if yes_branch is not None:
+                    yes_branch.finalize()
 
             new_statement = wrapStatementWithSideEffects(
                 new_node   = new_statement,
@@ -662,6 +686,8 @@ Removed conditional statement without effect."""
                 allow_none = True # surviving branch may empty
             )
 
+            del self.parent
+
             return new_statement, "new_statements", """\
 Condition for branch was predicted to be always %s.""" % choice
 
@@ -671,9 +697,7 @@ Condition for branch was predicted to be always %s.""" % choice
             # either.
             assert no_branch is not None
 
-            from .OperatorNodes import ExpressionOperationNOT
-
-            new_statement = StatementConditional(
+            new_statement = makeStatementConditional(
                 condition  = ExpressionOperationNOT(
                     operand    = condition,
                     source_ref = condition.getSourceReference()
@@ -683,6 +707,8 @@ Condition for branch was predicted to be always %s.""" % choice
                 source_ref = self.getSourceReference()
             )
 
+            del self.parent
+
             return new_statement, "new_statements", """\
 Empty 'yes' branch for conditional statement treated with inverted condition check."""
 
@@ -727,3 +753,41 @@ Empty 'yes' branch for conditional statement treated with inverted condition che
             return True
 
         return False
+
+    def getStatementNiceName(self):
+        return "branch statement"
+
+
+def makeStatementConditional(condition, yes_branch, no_branch, source_ref):
+    """ Create conditional statement, with yes_branch not being empty.
+
+        May have to invert condition to achieve that.
+    """
+
+    if yes_branch is None:
+        condition = ExpressionOperationNOT(
+            operand    = condition,
+            source_ref = condition.getSourceReference()
+        )
+
+        yes_branch, no_branch = no_branch, yes_branch
+
+    if yes_branch is not None and not yes_branch.isStatementsSequence():
+        yes_branch = StatementsSequence(
+            statements = (yes_branch,),
+            source_ref = source_ref
+        )
+
+    if no_branch is not None and not no_branch.isStatementsSequence():
+        no_branch = StatementsSequence(
+            statements = (no_branch,),
+            source_ref = source_ref
+        )
+
+    return StatementConditional(
+        condition  = condition,
+        yes_branch = yes_branch,
+        no_branch  = no_branch,
+        source_ref = source_ref
+
+    )
