@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -19,16 +19,22 @@
 
 """
 
-from .NodeBases import (
-    ExpressionChildrenHavingBase,
-    ExpressionMixin,
-    NodeBase,
-    StatementChildrenHavingBase
-)
+from .ExpressionBases import ExpressionBase, ExpressionChildrenHavingBase
+from .NodeBases import StatementBase, StatementChildrenHavingBase
 from .NodeMakingHelpers import makeStatementOnlyNodesFromExpressions
 
 
-class StatementRaiseException(StatementChildrenHavingBase):
+class StatementRaiseExceptionMixin(object):
+    @staticmethod
+    def isStatementAborting():
+        return True
+
+    @staticmethod
+    def isStatementRaiseException():
+        return True
+
+
+class StatementRaiseException(StatementRaiseExceptionMixin, StatementChildrenHavingBase):
     kind = "STATEMENT_RAISE_EXCEPTION"
 
     named_children = (
@@ -40,6 +46,8 @@ class StatementRaiseException(StatementChildrenHavingBase):
 
     def __init__(self, exception_type, exception_value, exception_trace,
                  exception_cause, source_ref):
+        assert exception_type is not None
+
         if exception_type is None:
             assert exception_value is None
 
@@ -72,25 +80,15 @@ class StatementRaiseException(StatementChildrenHavingBase):
         "exception_cause"
     )
 
-    def isStatementReraiseException(self):
-        return self.getExceptionType() is None
-
-    def isStatementAborting(self):
-        return True
-
-    @staticmethod
-    def isImplicit():
-        return False
-
-    def computeStatement(self, constraint_collection):
-        constraint_collection.onExpression(
+    def computeStatement(self, trace_collection):
+        trace_collection.onExpression(
             expression = self.getExceptionType(),
             allow_none = True
         )
         exception_type = self.getExceptionType()
 
         # TODO: Limit by type.
-        constraint_collection.onExceptionRaiseExit(BaseException)
+        trace_collection.onExceptionRaiseExit(BaseException)
 
         if exception_type is not None and \
            exception_type.willRaiseException(BaseException):
@@ -104,7 +102,7 @@ class StatementRaiseException(StatementChildrenHavingBase):
             return result, "new_raise", """\
 Explicit raise already raises implicitly building exception type."""
 
-        constraint_collection.onExpression(
+        trace_collection.onExpression(
             expression = self.getExceptionValue(),
             allow_none = True
         )
@@ -121,7 +119,7 @@ Explicit raise already raises implicitly building exception type."""
             return result, "new_raise", """\
 Explicit raise already raises implicitly building exception value."""
 
-        constraint_collection.onExpression(
+        trace_collection.onExpression(
             expression = self.getExceptionTrace(),
             allow_none = True
         )
@@ -140,7 +138,7 @@ Explicit raise already raises implicitly building exception value."""
             return result, "new_raise", """\
 Explicit raise already raises implicitly building exception traceback."""
 
-        constraint_collection.onExpression(
+        trace_collection.onExpression(
             expression = self.getExceptionCause(),
             allow_none = True
         )
@@ -161,19 +159,35 @@ Explicit raise already raises implicitly building exception cause."""
         return self, None, None
 
     def needsFrame(self):
-        return not self.isStatementReraiseException()
+        return True
+
+    def getStatementNiceName(self):
+        return "exception raise statement"
 
 
 class StatementRaiseExceptionImplicit(StatementRaiseException):
     kind = "STATEMENT_RAISE_EXCEPTION_IMPLICIT"
 
-    @staticmethod
-    def isStatementRaiseException():
-        return True
+    def getStatementNiceName(self):
+        return "implicit exception raise statement"
 
-    @staticmethod
-    def isImplicit():
-        return True
+
+class StatementReraiseException(StatementRaiseExceptionMixin, StatementBase):
+    kind = "STATEMENT_RERAISE_EXCEPTION"
+
+    def finalize(self):
+        del self.parent
+
+    def computeStatement(self, trace_collection):
+        trace_collection.onExceptionRaiseExit(BaseException)
+
+        return self, None, None
+
+    def needsFrame(self):
+        return False
+
+    def getStatementNiceName(self):
+        return "exception re-raise statement"
 
 
 class ExpressionRaiseException(ExpressionChildrenHavingBase):
@@ -204,10 +218,7 @@ class ExpressionRaiseException(ExpressionChildrenHavingBase):
     def willRaiseException(self, exception_type):
         # One thing is clear, it will raise. TODO: Match exception_type more
         # closely if it is predictable.
-        if exception_type is BaseException:
-            return True
-        else:
-            return False
+        return exception_type is BaseException
 
     getExceptionType = ExpressionChildrenHavingBase.childGetter(
         "exception_type"
@@ -216,11 +227,19 @@ class ExpressionRaiseException(ExpressionChildrenHavingBase):
         "exception_value"
     )
 
-    def computeExpression(self, constraint_collection):
+    def computeExpression(self, trace_collection):
         return self, None, None
 
-    def computeExpressionDrop(self, statement, constraint_collection):
-        result = StatementRaiseExceptionImplicit(
+    def computeExpressionDrop(self, statement, trace_collection):
+        result = self.asStatement()
+
+        del self.parent
+
+        return result, "new_raise", """\
+Propagated implicit raise expression to raise statement."""
+
+    def asStatement(self):
+        return StatementRaiseExceptionImplicit(
             exception_type  = self.getExceptionType(),
             exception_value = self.getExceptionValue(),
             exception_trace = None,
@@ -228,9 +247,6 @@ class ExpressionRaiseException(ExpressionChildrenHavingBase):
             source_ref      = self.getSourceReference()
         )
 
-        return result, "new_raise", """\
-Propagated implicit raise expression to raise statement."""
-
 
 class ExpressionBuiltinMakeException(ExpressionChildrenHavingBase):
     kind = "EXPRESSION_BUILTIN_MAKE_EXCEPTION"
@@ -260,7 +276,7 @@ class ExpressionBuiltinMakeException(ExpressionChildrenHavingBase):
 
     getArgs = ExpressionChildrenHavingBase.childGetter("args")
 
-    def computeExpression(self, constraint_collection):
+    def computeExpression(self, trace_collection):
         return self, None, None
 
     def mayRaiseException(self, exception_type):
@@ -271,16 +287,19 @@ class ExpressionBuiltinMakeException(ExpressionChildrenHavingBase):
         return False
 
 
-class ExpressionCaughtExceptionTypeRef(NodeBase, ExpressionMixin):
+class ExpressionCaughtExceptionTypeRef(ExpressionBase):
     kind = "EXPRESSION_CAUGHT_EXCEPTION_TYPE_REF"
 
     def __init__(self, source_ref):
-        NodeBase.__init__(
+        ExpressionBase.__init__(
             self,
             source_ref = source_ref
         )
 
-    def computeExpression(self, constraint_collection):
+    def finalize(self):
+        del self.parent
+
+    def computeExpressionRaw(self, trace_collection):
         # TODO: Might be predictable based on the exception handler this is in.
         return self, None, None
 
@@ -289,16 +308,19 @@ class ExpressionCaughtExceptionTypeRef(NodeBase, ExpressionMixin):
         return False
 
 
-class ExpressionCaughtExceptionValueRef(NodeBase, ExpressionMixin):
+class ExpressionCaughtExceptionValueRef(ExpressionBase):
     kind = "EXPRESSION_CAUGHT_EXCEPTION_VALUE_REF"
 
     def __init__(self, source_ref):
-        NodeBase.__init__(
+        ExpressionBase.__init__(
             self,
             source_ref = source_ref
         )
 
-    def computeExpression(self, constraint_collection):
+    def finalize(self):
+        del self.parent
+
+    def computeExpressionRaw(self, trace_collection):
         # TODO: Might be predictable based on the exception handler this is in.
         return self, None, None
 
@@ -307,16 +329,19 @@ class ExpressionCaughtExceptionValueRef(NodeBase, ExpressionMixin):
         return False
 
 
-class ExpressionCaughtExceptionTracebackRef(NodeBase, ExpressionMixin):
+class ExpressionCaughtExceptionTracebackRef(ExpressionBase):
     kind = "EXPRESSION_CAUGHT_EXCEPTION_TRACEBACK_REF"
 
     def __init__(self, source_ref):
-        NodeBase.__init__(
+        ExpressionBase.__init__(
             self,
             source_ref = source_ref
         )
 
-    def computeExpression(self, constraint_collection):
+    def finalize(self):
+        del self.parent
+
+    def computeExpressionRaw(self, trace_collection):
         return self, None, None
 
     def mayHaveSideEffects(self):
