@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -31,16 +31,18 @@ from logging import warning
 from nuitka.Builtins import builtin_names
 from nuitka.Constants import isConstant
 from nuitka.Options import isDebug, shallWarnImplicitRaises
+from nuitka.PythonVersions import python_version
 
 
 def makeConstantReplacementNode(constant, node):
-    from .ConstantRefNodes import ExpressionConstantRef
+    from .ConstantRefNodes import makeConstantRefNode
 
-    return ExpressionConstantRef(
+    return makeConstantRefNode(
         constant   = constant,
         source_ref = node.getSourceReference()
     )
 
+
 def makeRaiseExceptionReplacementExpression(expression, exception_type,
                                             exception_value):
     from .ExceptionNodes import ExpressionRaiseException
@@ -52,8 +54,8 @@ def makeRaiseExceptionReplacementExpression(expression, exception_type,
 
     if shallWarnImplicitRaises():
         warning(
-            "%s: Will always raise exception '%s(%s)'",
-            expression.getSourceReference().getAsString(),
+            '%s: Will always raise exception: "%s(%s)"',
+            source_ref.getAsString(),
             exception_type,
             exception_value
         )
@@ -73,6 +75,40 @@ def makeRaiseExceptionReplacementExpression(expression, exception_type,
     return result
 
 
+def makeRaiseExceptionReplacementStatement(statement, exception_type,
+                                           exception_value):
+    from .ExceptionNodes import StatementRaiseExceptionImplicit
+    from .BuiltinRefNodes import ExpressionBuiltinExceptionRef
+
+    source_ref = statement.getSourceReference()
+
+    assert type(exception_type) is str
+
+    if shallWarnImplicitRaises():
+        warning(
+            '%s: Will always raise exception: "%s(%s)"',
+            source_ref.getAsString(),
+            exception_type,
+            exception_value
+        )
+
+    result = StatementRaiseExceptionImplicit(
+        exception_type  = ExpressionBuiltinExceptionRef(
+            exception_name = exception_type,
+            source_ref     = source_ref
+        ),
+        exception_value = makeConstantReplacementNode(
+            constant = exception_value,
+            node     = statement
+        ),
+        exception_cause = None,
+        exception_trace = None,
+        source_ref      = source_ref
+    )
+
+    return result
+
+
 def makeRaiseExceptionReplacementExpressionFromInstance(expression, exception):
     assert isinstance(exception, Exception)
 
@@ -90,6 +126,79 @@ def makeRaiseExceptionReplacementExpressionFromInstance(expression, exception):
     )
 
 
+def makeRaiseExceptionExpressionFromTemplate(exception_type, template,
+                                             template_args, source_ref):
+    from .ExceptionNodes import ExpressionRaiseException
+    from .BuiltinRefNodes import ExpressionBuiltinExceptionRef
+    from .OperatorNodes import makeBinaryOperationNode
+    from .ConstantRefNodes import makeConstantRefNode
+
+    return ExpressionRaiseException(
+        exception_type  = ExpressionBuiltinExceptionRef(
+            exception_name = exception_type,
+            source_ref     = source_ref
+        ),
+        exception_value = makeBinaryOperationNode(
+            operator   = "Mod",
+            left       =  makeConstantRefNode(
+                constant      = template,
+                source_ref    = source_ref,
+                user_provided = True
+            ),
+            right      = template_args,
+            source_ref = source_ref
+        ),
+        source_ref      = source_ref
+    )
+
+def makeRaiseTypeErrorExceptionReplacementFromTemplateAndValue(template,
+                                                               operation,
+                                                               original_node,
+                                                               value_node):
+    shape = value_node.getTypeShape()
+
+    type_name = shape.getTypeName()
+
+    if type_name is not None:
+        result = makeRaiseExceptionReplacementExpressionFromInstance(
+            expression = original_node,
+            exception  = TypeError(template % type_name if '%' in template else template)
+        )
+
+        result = wrapExpressionWithNodeSideEffects(
+            new_node = result,
+            old_node = value_node
+        )
+
+    else:
+        from .AttributeNodes import ExpressionAttributeLookup
+        from .TypeNodes import ExpressionBuiltinType1
+
+        source_ref = original_node.getSourceReference()
+
+        result = makeRaiseExceptionExpressionFromTemplate(
+            exception_type = "TypeError",
+            template       = "object of type '%s' has no len()",
+            template_args  = ExpressionAttributeLookup(
+                source         = ExpressionBuiltinType1(
+                    value      = value_node.makeClone(),
+                    source_ref = source_ref
+                ),
+                attribute_name = "__name__",
+                source_ref     = source_ref
+            ),
+            source_ref     = source_ref
+        )
+
+        type_name = shape.__name__
+
+    return result, "new_raise", "Raising for use of '%s' on %s '%s'." % (
+        operation,
+        "type" if type_name is not None else "shape",
+        type_name
+    )
+
+
 def makeCompileTimeConstantReplacementNode(value, node):
     # This needs to match code in isCompileTimeConstantValue
     if isConstant(value):
@@ -99,9 +208,9 @@ def makeCompileTimeConstantReplacementNode(value, node):
         )
     elif type(value) is type:
         if value.__name__ in builtin_names:
-            from .BuiltinRefNodes import ExpressionBuiltinRef
+            from .BuiltinRefNodes import makeExpressionBuiltinRef
 
-            return ExpressionBuiltinRef(
+            return makeExpressionBuiltinRef(
                 builtin_name = value.__name__,
                 source_ref   = node.getSourceReference()
             )
@@ -110,13 +219,14 @@ def makeCompileTimeConstantReplacementNode(value, node):
     else:
         return node
 
+
 def getComputationResult(node, computation, description):
     """ With a computation function, execute it and return constant result or
         exception node.
 
     """
 
-    # Try and turn raised exceptions into static raises. pylint: disable=W0703
+    # Try and turn raised exceptions into static raises. pylint: disable=broad-except
     try:
         result = computation()
     except Exception as e:
@@ -182,6 +292,7 @@ def makeStatementsSequenceReplacementNode(statements, node):
         source_ref = node.getSourceReference()
     )
 
+
 def convertNoneConstantToNone(node):
     if node is None:
         return None
@@ -190,18 +301,30 @@ def convertNoneConstantToNone(node):
     else:
         return node
 
+
 def wrapExpressionWithSideEffects(side_effects, old_node, new_node):
     assert new_node.isExpression()
 
     from .SideEffectNodes import ExpressionSideEffects
 
     if side_effects:
-        new_node = ExpressionSideEffects(
-            expression   = new_node,
-            side_effects = side_effects,
-            source_ref   = old_node.getSourceReference()
+        side_effects = sum(
+            (
+                side_effect.extractSideEffects()
+                for side_effect in
+                side_effects
+                if side_effect.mayHaveSideEffects()
+            ),
+            ()
         )
 
+        if side_effects:
+            new_node = ExpressionSideEffects(
+                expression   = new_node,
+                side_effects = side_effects,
+                source_ref   = old_node.getSourceReference()
+            )
+
     return new_node
 
 def wrapExpressionWithNodeSideEffects(new_node, old_node):
@@ -254,7 +377,7 @@ def makeStatementOnlyNodesFromExpressions(expressions):
     if not statements:
         return None
     elif len(statements) == 1:
-        return statements[ 0 ]
+        return statements[0]
     else:
         return StatementsSequence(
             statements = statements,
@@ -266,7 +389,9 @@ def makeComparisonNode(left, right, comparator, source_ref):
     from .ComparisonNodes import (
         ExpressionComparison,
         ExpressionComparisonIs,
-        ExpressionComparisonIsNOT
+        ExpressionComparisonIsNOT,
+        ExpressionComparisonIn,
+        ExpressionComparisonNOTIn
     )
 
     if comparator == "Is":
@@ -277,10 +402,22 @@ def makeComparisonNode(left, right, comparator, source_ref):
         )
     elif comparator == "IsNot":
         result = ExpressionComparisonIsNOT(
-                left       = left,
-                right      = right,
-                source_ref = source_ref
-            )
+            left       = left,
+            right      = right,
+            source_ref = source_ref
+        )
+    elif comparator == "In":
+        result = ExpressionComparisonIn(
+            left       = left,
+            right      = right,
+            source_ref = source_ref
+        )
+    elif comparator == "NotIn":
+        result = ExpressionComparisonNOTIn(
+            left       = left,
+            right      = right,
+            source_ref = source_ref
+        )
     else:
         result = ExpressionComparison(
             left       = left,
@@ -308,25 +445,46 @@ def makeVariableRefNode(variable, source_ref):
         from .VariableRefNodes import ExpressionVariableRef
 
         return ExpressionVariableRef(
-            variable_name = variable.getName(),
-            variable      = variable,
-            source_ref    = source_ref
-        )
-
-
-def makeVariableTargetRefNode(variable, source_ref):
-    if variable.isTempVariable():
-        from .AssignNodes import ExpressionTargetTempVariableRef
-
-        return ExpressionTargetTempVariableRef(
             variable   = variable,
             source_ref = source_ref
         )
-    else:
-        from .AssignNodes import ExpressionTargetVariableRef
 
-        return ExpressionTargetVariableRef(
-            variable_name = variable.getName(),
-            variable      = variable,
-            source_ref    = source_ref
+
+def makeExpressionBuiltinLocals(provider, source_ref):
+    while provider.isExpressionOutlineFunction():
+        provider = provider.getParentVariableProvider()
+
+    if provider.isCompiledPythonModule():
+        from .GlobalsLocalsNodes import ExpressionBuiltinGlobals
+
+        return ExpressionBuiltinGlobals(
+            source_ref = source_ref
         )
+    else:
+        from .GlobalsLocalsNodes import (
+            ExpressionBuiltinLocalsCopy,
+            ExpressionBuiltinLocalsRef,
+            ExpressionBuiltinLocalsUpdated
+        )
+
+        if provider.isExpressionClassBody():
+            return ExpressionBuiltinLocalsRef(
+                locals_scope = provider.getFunctionLocalsScope(),
+                source_ref   = source_ref
+            )
+        elif python_version >= 300 or provider.isUnoptimized():
+            assert provider.getFunctionLocalsScope(), provider
+
+            return ExpressionBuiltinLocalsUpdated(
+                locals_scope = provider.getFunctionLocalsScope(),
+                source_ref   = source_ref
+            )
+        else:
+            # TODO: Make this not true, there ought to be always a locals
+            # scope.
+            assert not provider.getFunctionLocalsScope(), provider
+
+            return ExpressionBuiltinLocalsCopy(
+                locals_scope = provider.getFunctionLocalsScope(),
+                source_ref   = source_ref
+            )
