@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -29,8 +29,8 @@ from nuitka.plugins.PluginBase import NuitkaPluginBase
 from nuitka.utils import Utils
 
 
-class NuitkaPluginMultiprocessingWorkaorunds(NuitkaPluginBase):
-    """ This is to make multiprocess work with Nuitka and use compiled code.
+class NuitkaPluginMultiprocessingWorkarounds(NuitkaPluginBase):
+    """ This is to make multiprocessing work with Nuitka and use compiled code.
 
         When running in accelerated mode, it's not good to fork a new Python
         instance to run other code, as that won't be accelerated. And when
@@ -38,7 +38,7 @@ class NuitkaPluginMultiprocessingWorkaorunds(NuitkaPluginBase):
         same principle.
 
         So by default, this module is on and works around the behavior of the
-        "multiprocess.forking" expectations.
+        "multiprocessing.forking/multiprocessing.spawn" expectations.
     """
     plugin_name = "multiprocessing"
 
@@ -49,7 +49,7 @@ class NuitkaPluginMultiprocessingWorkaorunds(NuitkaPluginBase):
     def createPreModuleLoadCode(module):
         full_name = module.getFullName()
 
-        if full_name == "multiprocessing.forking":
+        if full_name in ("multiprocessing.forking", "multiprocessing.spawn"):
             code = """\
 import sys
 sys.frozen = 1
@@ -65,9 +65,9 @@ Monkey patching "multiprocessing" load environment."""
     def createPostModuleLoadCode(module):
         full_name = module.getFullName()
 
-        if full_name == "multiprocessing.forking":
+        if full_name in ("multiprocessing.forking", "multiprocessing.reduction"):
             code = """\
-from multiprocessing.forking import ForkingPickler
+from %s import ForkingPickler
 
 class C:
    def f():
@@ -79,15 +79,65 @@ def _reduce_compiled_method(m):
     else:
         return getattr, (m.im_self, m.im_func.__name__)
 
-print type(_reduce_compiled_method)
+ForkingPickler.register(type(C().f), _reduce_compiled_method)
 ForkingPickler.register(type(C.f), _reduce_compiled_method)
 """
+            code %= full_name
+
             return code, """\
 Monkey patching "multiprocessing" for compiled methods."""
 
-
         return None, None
 
+    @staticmethod
+    def _addSlaveMainModule(root_module):
+        from nuitka.tree.Building import CompiledPythonModule, readSourceCodeFromFilename, createModuleTree
+        from nuitka.ModuleRegistry import addRootModule
+        from nuitka.plugins.Plugins import Plugins
+        from sys import hexversion
+        # First, build the module node and then read again from the
+        # source code.
+        module_name = "__parents_main__"
+        source_ref = root_module.getSourceReference()
+
+        mode = Plugins.decideCompilation(module_name, source_ref)
+
+        slave_main_module = CompiledPythonModule(
+            name         = module_name,
+            package_name = None,
+            mode         = mode,
+            future_spec  = None,
+            source_ref   = root_module.getSourceReference()
+        )
+
+        source_code = readSourceCodeFromFilename(
+            "__parents_main__",
+            root_module.getFilename()
+        )
+
+        # For the call stack, this may look bad or different to what
+        # CPython does. Using the "__import__" built-in to not spoil
+        # or use the module namespace. The forking module was split up
+        # into multiple modules in Python 3.4.0.a2
+        if hexversion >= 0x030400A2:
+            source_code += """
+__import__("sys").modules["__main__"] = __import__("sys").modules[__name__]
+__import__("multiprocessing.spawn").spawn.freeze_support()"""
+        else:
+            source_code += """
+__import__("sys").modules["__main__"] = __import__("sys").modules[__name__]
+__import__("multiprocessing.forking").forking.freeze_support()"""
+
+        createModuleTree(
+            module      = slave_main_module,
+            source_ref  = root_module.getSourceReference(),
+            source_code = source_code,
+            is_main     = False
+        )
+
+        # This is an alternative entry point of course.
+        addRootModule(slave_main_module)
+
     def onModuleEncounter(self, module_filename, module_name, module_package,
                           module_kind):
         if module_name == "multiprocessing" and \
@@ -95,48 +145,20 @@ Monkey patching "multiprocessing" for compiled methods."""
            and not self.multiprocessing_added:
             self.multiprocessing_added = True
 
-            from nuitka.ModuleRegistry import getRootModules, addRootModule
-            from nuitka.tree.Building import PythonModule, readSourceCodeFromFilename, createModuleTree
+            from nuitka.ModuleRegistry import getRootModules
 
             for root_module in getRootModules():
                 if root_module.isMainModule():
-                    # First, build the module node and then read again from the
-                    # source code.
-
-                    slave_main_module = PythonModule(
-                        name         = "__parents_main__",
-                        package_name = None,
-                        source_ref   = root_module.getSourceReference()
-                    )
-
-                    source_code = readSourceCodeFromFilename(
-                        "__parents_main__",
-                        root_module.getFilename()
-                    )
-
-                    # For the call stack, this may look bad or different to what
-                    # CPython does. Using the "__import__" built-in to not spoil
-                    # or use the module namespace.
-                    source_code += """
-__import__("sys").modules["__main__"] = __import__("sys").modules[__name__]
-__import__("multiprocessing.forking").forking.main()"""
-
-                    createModuleTree(
-                        module      = slave_main_module,
-                        source_ref  = root_module.getSourceReference(),
-                        source_code = source_code,
-                        is_main     = False
-                    )
-
-                    # This is an alternative entry point of course.
-                    addRootModule(slave_main_module)
-
+                    self._addSlaveMainModule(root_module)
                     break
             else:
                 assert False
 
+        if module_package == "multiprocessing" and module_name in ("forking", "spawn"):
+            return True, "Multiprocessing plugin needs this to monkey patch it."
 
-class NuitkaPluginDetectorMultiprocessingWorkaorunds(NuitkaPluginBase):
+
+class NuitkaPluginDetectorMultiprocessingWorkarounds(NuitkaPluginBase):
     plugin_name = "multiprocessing"
 
     @staticmethod
