@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -70,7 +70,7 @@ from nuitka.nodes.BuiltinFormatNodes import (
 from nuitka.nodes.BuiltinTypeNodes import ExpressionBuiltinStr
 from nuitka.nodes.ConditionalNodes import (
     ExpressionConditional,
-    StatementConditional
+    makeStatementConditional
 )
 from nuitka.nodes.ConstantRefNodes import (
     ExpressionConstantEllipsisRef,
@@ -78,7 +78,10 @@ from nuitka.nodes.ConstantRefNodes import (
     makeConstantRefNode
 )
 from nuitka.nodes.CoroutineNodes import ExpressionAsyncWait
-from nuitka.nodes.ExceptionNodes import StatementRaiseException
+from nuitka.nodes.ExceptionNodes import (
+    StatementRaiseException,
+    StatementReraiseException
+)
 from nuitka.nodes.GeneratorNodes import StatementGeneratorReturn
 from nuitka.nodes.LoopNodes import StatementLoopBreak, StatementLoopContinue
 from nuitka.nodes.ModuleAttributeNodes import ExpressionModuleAttributeFileRef
@@ -199,7 +202,7 @@ def buildConditionNode(provider, node, source_ref):
     # "elif", because that's already dealt with by module "ast", which turns it
     # into nested conditional statements.
 
-    return StatementConditional(
+    return makeStatementConditional(
         condition  = buildNode(provider, node.test, source_ref),
         yes_branch = buildStatementsNode(
             provider   = provider,
@@ -282,31 +285,40 @@ def buildRaiseNode(provider, node, source_ref):
         exception_trace = None
         exception_cause = buildNode(provider, node.cause, source_ref, allow_none = True)
 
-    result = StatementRaiseException(
-        exception_type  = exception_type,
-        exception_value = exception_value,
-        exception_trace = exception_trace,
-        exception_cause = exception_cause,
-        source_ref      = source_ref
-    )
+    if exception_type is None:
+        assert exception_value is None
+        assert exception_trace is None
+        assert exception_cause is None
 
-    if exception_cause is not None:
-        result.setCompatibleSourceReference(
-            source_ref = exception_cause.getCompatibleSourceReference()
+        result = StatementReraiseException(
+            source_ref = source_ref
         )
-    elif exception_trace is not None:
-        result.setCompatibleSourceReference(
-            source_ref = exception_trace.getCompatibleSourceReference()
-        )
-    elif exception_value is not None:
-        result.setCompatibleSourceReference(
-            source_ref = exception_value.getCompatibleSourceReference()
-        )
-    elif exception_type is not None:
-        result.setCompatibleSourceReference(
-            source_ref = exception_type.getCompatibleSourceReference()
+    else:
+        result = StatementRaiseException(
+            exception_type  = exception_type,
+            exception_value = exception_value,
+            exception_trace = exception_trace,
+            exception_cause = exception_cause,
+            source_ref      = source_ref
         )
 
+        if exception_cause is not None:
+            result.setCompatibleSourceReference(
+                source_ref = exception_cause.getCompatibleSourceReference()
+            )
+        elif exception_trace is not None:
+            result.setCompatibleSourceReference(
+                source_ref = exception_trace.getCompatibleSourceReference()
+            )
+        elif exception_value is not None:
+            result.setCompatibleSourceReference(
+                source_ref = exception_value.getCompatibleSourceReference()
+            )
+        elif exception_type is not None:
+            result.setCompatibleSourceReference(
+                source_ref = exception_type.getCompatibleSourceReference()
+            )
+
     return result
 
 
@@ -495,7 +507,7 @@ def buildReturnNode(provider, node, source_ref):
     expression = buildNode(provider, node.value, source_ref, allow_none = True)
 
     if provider.isExpressionGeneratorObjectBody():
-        if expression is not None and python_version < 330:
+        if expression is not None and python_version < 300:
             SyntaxErrors.raiseSyntaxError(
                 "'return' with argument inside generator",
                 source_ref.atColumnNumber(node.col_offset)
@@ -823,8 +835,7 @@ def buildParseTree(provider, source_code, source_ref, is_module, is_main):
             )
         )
 
-    needs__initializing__ = not provider.isMainModule() and \
-      (python_version >= 330 and python_version < 340)
+    needs__initializing__ = not provider.isMainModule() and 300 <= python_version < 340
 
     if needs__initializing__:
         # Set "__initializing__" at the beginning to True
